#include "gpio.hpp"
#include "output-gpio-init.hpp"

#ifndef OUTPUT_GPIO_H
#define OUTPUT_GPIO_H

namespace stm32f072xb {

template <Port T>
class OutputGpio : public Gpio<T> {
   private:
	FlagStatus _status;

   public:
	OutputGpio(const OutputGpioInit& gpioInit) : Gpio<T>(gpioInit) {
		CLEAR_BIT(this->getGpio()->MODER, 0x3UL << (this->getPin() * 2));
		SET_BIT(this->getGpio()->MODER, 0x1UL << (this->getPin() * 2));
		CLEAR_BIT(this->getGpio()->OTYPER, 0x1UL << (this->getPin()));
	}

	void setPin(const FlagStatus& status) {
		_status = status;
		(status == FlagStatus::SET) ? SET_BIT(this->getGpio()->ODR, 1 << this->getPin())
		                            : CLEAR_BIT(this->getGpio()->ODR, 1 << this->getPin());
	}

	void toggle() {
		_status = (_status == FlagStatus::SET) ? FlagStatus::RESET : FlagStatus::SET;
		setPin(_status);
	}
};

}  // namespace stm32f072xb

#endif
